/* =================================================================================
 *  alberi.h
 * =================================================================================
 * Contiene la definizione di albero binario (di interi), ed i prototipi delle
 * seguenti funzioni e procedure su alberi binari:
 * + costruzione di un albero binario inserito da input
 * + visita / stampa di un albero
 * + calcolo del numero delle foglie dell'albero (versione funzionale)
 * + calcolo del numero di nodi (versione funzionale e procedurale)
 * + calcolo dell'altezza (versione funzionale e procedurale)
 * + calcolo del livello della foglia piu' a sinistra (versione funzionale e procedurale)
 * + stampa in output indentata di un albero binario
 * + verifica se l'albero e' vuoto
 *
 *  Authors:     Emanuela Fachini (fachini@di.uniroma1.it)
 *               Irene Finocchi   (finocchi@di.uniroma1.it)
 *               Chiara Petrioli  (petrioli@di.uniroma1.it)
 *  Last change: 2 Aprile 2005
 *  Changed by:  Irene Finocchi
*/

#ifndef ALBERI_H
#define ALBERI_H

#include <stdio.h>
#include<stdlib.h>

struct nodoalbero{
	struct nodoalbero *rPtr;
	int elem;
	struct nodoalbero *lPtr;
};

typedef struct nodoalbero TREENODE;
typedef TREENODE *TreePtr;


/* Costruzione di un albero binario inserito da input. Si assume che gli interi
 * contenuti nei nodi dell'albero siano inseriti da input in preordine
 * Il formato dell'input saranno coppie di interi in cui il primo valore
 * corrisponde al valore da inserire nel nodo, il secondo valore indica il
 * numero dei suoi figli (0 nessun figlio, 1 solo il figlio sinistro, -1 solo
 * il figlio destro, 2 entrambi i figli). I nodi sono inseriti i preordine (prima la
 * radice), quindi i nodi del sottoalbero sinistro, quindi i nodi del sottoalbero
 * destro. Ad esempio:
 * 3 2
 * 1 0
 * 4 2
 * 5 0
 * 7 0
 *
 * costruisce l'albero binario
 *    3
 *  1	 4
 *     5   7
 * La seguente procedura costruisce l'albero binario */
TreePtr creaalbero();

/* Verifica se l'albero e' vuoto */
int alberovuoto(TreePtr);

/* Calcola il numero di nodi dell'albero: versione funzionale e procedurale */
int numNodi(TreePtr tPtr);
void numNodi2(TreePtr tPtr, int *n);

/* Calcola l'altezza dell'albero: versione funzionale e procedurale */
int max (int x, int y);
int alt(TreePtr tPtr);
void alt2(TreePtr tPtr, int* h);

/* Calcola la foglia piu' a sinistra dell'albero: versione funzionale e procedurale */
int livFoSin(TreePtr tPtr); 
void livFoSin2(TreePtr tPtr, int* liv); 

/* stampa in inorder i valori contenuti nell'albero */
void stampaAlberoInorder(TreePtr);

/* calcola il numero di nodi dell'albero al livello specificato */ 
int numelemlivello (TreePtr, int, int);

/* calcola il numero di foglie dell'albero */
int numfoglie(TreePtr);

/* stampa l'albero indentato */
void stampaIndentata(TreePtr, int);

#endif
